@@ -2,13 +2,14 @@
 #include "xyzsh/xyzsh.h"
 #include <string.h>
 #include <stdio.h>
+#include <glob.h>
 
 sCommand* sCommand_new(sStatment* statment)
 {
     sCommand* self = statment->mCommands + statment->mCommandsNum;
 
     self->mArgs = MALLOC(sizeof(char*)*3);
-    self->mArgsEnv = MALLOC(sizeof(int)*3);
+    self->mArgsFlags = MALLOC(sizeof(int)*3);
     self->mArgsSize = 3;
     self->mArgsNum = 0;
 
@@ -179,7 +180,7 @@ void sStatment_sweep_runtime_info(sStatment* self)
     }
 }
 
-void sCommand_add_redirect_to_command(sCommand* self, MANAGED char* name, int redirect)
+void sCommand_add_redirect_to_command(sCommand* self, MANAGED char* name, BOOL env, BOOL glob, int redirect)
 {
     if(self->mRedirectsNum >= self->mRedirectsSize) {
         if(self->mRedirectsFileNames == NULL) {
@@ -199,7 +200,7 @@ void sCommand_add_redirect_to_command(sCommand* self, MANAGED char* name, int re
     }
 
     self->mRedirectsFileNames[self->mRedirectsNum] = MANAGED name;
-    self->mRedirects[self->mRedirectsNum++] = redirect;
+    self->mRedirects[self->mRedirectsNum++] = redirect | (env ? REDIRECT_ENV:0) | (glob ? REDIRECT_GLOB: 0);
 }
 
 typedef struct {
@@ -260,14 +261,14 @@ static void sCommand_copy_deeply(sCommand* dest, sCommand* src)
     dest->mKind = src->mKind;
 
     dest->mArgs = MALLOC(sizeof(char*)*src->mArgsSize);
-    dest->mArgsEnv = MALLOC(sizeof(int)*src->mArgsSize);
+    dest->mArgsFlags = MALLOC(sizeof(int)*src->mArgsSize);
     dest->mArgsNum = src->mArgsNum;
     dest->mArgsSize = src->mArgsSize;
 
     int i;
     for(i=0; i<src->mArgsNum; i++) {
         dest->mArgs[i] = STRDUP(src->mArgs[i]);
-        dest->mArgsEnv[i] = src->mArgsEnv[i];
+        dest->mArgsFlags[i] = src->mArgsFlags[i];
     }
     dest->mArgs[i] = NULL;
 
@@ -320,6 +321,7 @@ static void sCommand_copy_deeply(sCommand* dest, sCommand* src)
     for(i=0; i<src->mRedirectsNum; i++) {
         dest->mRedirects[i] = src->mRedirects[i];
     }
+
     dest->mRedirectsSize = src->mRedirectsSize;
     dest->mRedirectsNum = src->mRedirectsNum;
 
@@ -347,14 +349,14 @@ static void sCommand_copy_deeply_stack(sCommand* dest, sCommand* src)
     dest->mKind = src->mKind;
 
     dest->mArgs = MALLOC(sizeof(char*)*src->mArgsSize);
-    dest->mArgsEnv = MALLOC(sizeof(int)*src->mArgsSize);
+    dest->mArgsFlags = MALLOC(sizeof(int)*src->mArgsSize);
     dest->mArgsNum = src->mArgsNum;
     dest->mArgsSize = src->mArgsSize;
 
     int i;
     for(i=0; i<src->mArgsNum; i++) {
         dest->mArgs[i] = STRDUP(src->mArgs[i]);
-        dest->mArgsEnv[i] = src->mArgsEnv[i];
+        dest->mArgsFlags[i] = src->mArgsFlags[i];
     }
     dest->mArgs[i] = NULL;
 
@@ -435,14 +437,14 @@ static void sCommand_copy_deeply_malloc(sCommand* dest, sCommand* src)
     dest->mKind = src->mKind;
 
     dest->mArgs = MALLOC(sizeof(char*)*src->mArgsSize);
-    dest->mArgsEnv = MALLOC(sizeof(int)*src->mArgsSize);
+    dest->mArgsFlags = MALLOC(sizeof(int)*src->mArgsSize);
     dest->mArgsNum = src->mArgsNum;
     dest->mArgsSize = src->mArgsSize;
 
     int i;
     for(i=0; i<src->mArgsNum; i++) {
         dest->mArgs[i] = STRDUP(src->mArgs[i]);
-        dest->mArgsEnv[i] = src->mArgsEnv[i];
+        dest->mArgsFlags[i] = src->mArgsFlags[i];
     }
     dest->mArgs[i] = NULL;
 
@@ -525,7 +527,7 @@ void sCommand_delete(sCommand* self)
         FREE(self->mArgs[i]);
     }
     FREE(self->mArgs);
-    FREE(self->mArgsEnv);
+    FREE(self->mArgsFlags);
     for(i=0; i<self->mArgsNumRuntime; i++) {
         FREE(self->mArgsRuntime[i]);
     }
@@ -573,7 +575,7 @@ void sCommand_delete_malloc(sCommand* self)
         FREE(self->mArgs[i]);
     }
     FREE(self->mArgs);
-    FREE(self->mArgsEnv);
+    FREE(self->mArgsFlags);
     for(i=0; i<self->mArgsNumRuntime; i++) {
         FREE(self->mArgsRuntime[i]);
     }
@@ -629,22 +631,22 @@ void sCommand_view(sCommand* self)
     }
 }
 
-void sCommand_add_arg_to_command(sCommand* self, MANAGED char* buf, int env)
+void sCommand_add_arg_to_command(sCommand* self, MANAGED char* buf, int env, int glob)
 {
     if(self->mArgsNum+1 >= self->mArgsSize) {
         const int arg_size = self->mArgsSize;
 
         int new_size = self->mArgsSize * 2;
         self->mArgs = REALLOC(self->mArgs, sizeof(char*)*new_size);
-        self->mArgsEnv = REALLOC(self->mArgsEnv,sizeof(int)*new_size);
+        self->mArgsFlags = REALLOC(self->mArgsFlags,sizeof(int)*new_size);
         memset(self->mArgs + self->mArgsSize, 0, sizeof(char) * (new_size - self->mArgsSize));
-        memset(self->mArgsEnv + self->mArgsSize, 0, sizeof(int)* (new_size - self->mArgsSize));
+        memset(self->mArgsFlags + self->mArgsSize, 0, sizeof(int)* (new_size - self->mArgsSize));
         self->mArgsSize = new_size;
 
     }
 
     self->mArgs[self->mArgsNum] = buf;
-    self->mArgsEnv[self->mArgsNum++] = env;
+    self->mArgsFlags[self->mArgsNum++] = (env ? ARG_ENV:0) | (glob ? ARG_GLOB:0);
     self->mArgs[self->mArgsNum] = NULL;
 }
 
@@ -1103,6 +1105,47 @@ static BOOL expand_variable(sCommand* command, sObject* object, sBuf* buf, sRunI
     return TRUE;
 }
 
+static BOOL expand_glob(char* buf, ALLOC char** head, ALLOC char*** tails, sCommand* command, char* sname, int sline)
+{
+    glob_t result;
+    int rc = glob(buf, 0, NULL, &result);
+
+    if(rc == GLOB_NOSPACE) {
+        err_msg("read_one_argument: out of space during glob operation", sname, sline, buf);
+        globfree(&result);
+        return FALSE;
+    }
+    else if(rc == GLOB_NOMATCH) {
+        *head = STRDUP(buf);
+
+        *tails = MALLOC(sizeof(char*)*result.gl_pathc);
+        **tails = NULL;
+    }
+    else {
+        if(result.gl_pathc > 0) {
+            *head = STRDUP(result.gl_pathv[0]);
+
+            *tails = MALLOC(sizeof(char*)*result.gl_pathc);
+
+            int i;
+            for(i=1; i<result.gl_pathc; i++) {
+                (*tails)[i-1] = STRDUP(result.gl_pathv[i]);
+            }
+            (*tails)[i-1] = 0;
+        }
+        else {
+            *head = STRDUP(buf);
+
+            *tails = MALLOC(sizeof(char*)*result.gl_pathc);
+            **tails = NULL;
+        }
+    }
+
+    globfree(&result);
+
+    return TRUE;
+}
+
 // if return is FALSE, no allocate result
 static BOOL expand_env_to_command(ALLOC char** result, char* str, sCommand* command, sRunInfo* runinfo, sObject* nextin, BOOL* expand_double_dollar)
 {
@@ -1218,7 +1261,13 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
             return FALSE;
         }
 
-        if(command->mArgsEnv[*i+1]) {
+        /// environment or environment and glob ///
+        if(command->mArgsFlags[*i+1] & ARG_GLOB) {
+            err_msg("can't expand glob with option", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+            FREE(arg);
+            return FALSE;
+        }
+        else if(command->mArgsFlags[*i+1] & ARG_ENV) {
             char* expanded_str;
             BOOL expand_double_dollar;
             if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[*i+1], command, runinfo, nextin, &expand_double_dollar)) {
@@ -1229,6 +1278,7 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
             if(expand_double_dollar) {
                 err_msg("invalid expanding varibale block", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
                 FREE(expanded_str);
+                FREE(arg);
                 return FALSE;
             }
 
@@ -1250,7 +1300,8 @@ static BOOL get_option(MANAGED char* arg, sCommand* command, sObject* fun, sObje
                 FREE(arg);
                 return FALSE;
             }
-            else if(!sCommand_put_option_with_argument(command, MANAGED arg, MANAGED STRDUP(command->mArgs[*i+1])))
+
+            if(!sCommand_put_option_with_argument(command, MANAGED arg, MANAGED STRDUP(command->mArgs[*i+1])))
             {
                 char buf[BUFSIZ];
                 snprintf(buf, BUFSIZ, "option number max (%s)\n", arg);
@@ -1290,7 +1341,12 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sObje
     for(i=0; i < command->mArgsNum; i++) {
         /// options ///
         if(command->mArgs[i][0] == PARSER_MAGIC_NUMBER_OPTION) {
-            if(command->mArgsEnv[i]) {
+            if(command->mArgsFlags[i] & ARG_GLOB) {
+                err_msg("can't expand glob in option", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                return FALSE;
+            }
+
+            if(command->mArgsFlags[i] & ARG_ENV) {
                 char* expanded_str;
                 BOOL expand_double_dollar;
                 if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[i], command, runinfo, nextin, &expand_double_dollar)) {
@@ -1332,24 +1388,71 @@ BOOL sCommand_expand_env(sCommand* command, sObject* fun, sObject* nextin, sObje
             }
         }
         /// variable ///
-        else if(command->mArgsEnv[i]) {
-            char* expanded_str;
-            BOOL expand_double_dollar;
-            if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[i], command, runinfo, nextin, &expand_double_dollar)) {
-                return FALSE;
-            }
+        else if(command->mArgsFlags[i] & ARG_ENV) {
+            /// variable and glob ///
+            if(command->mArgsFlags[i] & ARG_GLOB) {
+                char* expanded_str;
+                BOOL expand_double_dollar;
+                if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[i], command, runinfo, nextin, &expand_double_dollar)) {
+                    return FALSE;
+                }
 
-            if(expand_double_dollar) {
-                if(expanded_str[0] == 0) {
+                if(expand_double_dollar) {
                     FREE(expanded_str);
+                    err_msg("can't expand glob with double dollar environment block", runinfo->mSName, runinfo->mSLine, command->mArgs[0]);
+                    return FALSE;
+                }
+                else {
+                    char* head;
+                    char** tails;
+                    if(!expand_glob(expanded_str, ALLOC &head, ALLOC &tails, command, runinfo->mSName, runinfo->mSLine))
+                    {
+                        return FALSE;
+                    }
+                    sCommand_add_arg_to_command2(command, MANAGED head);
+                    char** p = tails;
+                    while(*p) {
+                        sCommand_add_arg_to_command2(command, MANAGED *p);
+                        p++;
+                    }
+                    FREE(tails);
+                }
+            }
+            else {
+                char* expanded_str;
+                BOOL expand_double_dollar;
+                if(!expand_env_to_command(ALLOC &expanded_str, command->mArgs[i], command, runinfo, nextin, &expand_double_dollar)) {
+                    return FALSE;
+                }
+
+                if(expand_double_dollar) {
+                    if(expanded_str[0] == 0) {
+                        FREE(expanded_str);
+                    }
+                    else {
+                        sCommand_add_arg_to_command2(command, MANAGED expanded_str);
+                    }
                 }
                 else {
                     sCommand_add_arg_to_command2(command, MANAGED expanded_str);
                 }
             }
-            else {
-                sCommand_add_arg_to_command2(command, MANAGED expanded_str);
+        }
+        /// glob
+        else if(command->mArgsFlags[i] & ARG_GLOB) {
+            char* head;
+            char** tails;
+            if(!expand_glob(command->mArgs[i], ALLOC &head, ALLOC &tails, command, runinfo->mSName, runinfo->mSLine))
+            {
+                return FALSE;
             }
+            sCommand_add_arg_to_command2(command, MANAGED head);
+            char** p = tails;
+            while(*p) {
+                sCommand_add_arg_to_command2(command, MANAGED *p);
+                p++;
+            }
+            FREE(tails);
         }
         /// normal ///
         else {
@@ -1367,19 +1470,10 @@ BOOL sCommand_expand_env_redirect(sCommand* command, sObject* nextin, sRunInfo*
 
         int i;
         for(i=0; i<command->mRedirectsNum; i++) {
-            char* fname = command->mRedirectsFileNames[i];
-            BOOL env = FALSE;
-            char* p = fname;
-            while(*p) {
-                if(*p == PARSER_MAGIC_NUMBER_ENV) {
-                    env = TRUE;
-                }
-                p++;
-            }
-            if(env) {
+            if(command->mRedirects[i] & REDIRECT_ENV) {
                 char* expanded_str;
                 BOOL expand_double_dollar;
-                if(!expand_env_to_command(ALLOC &expanded_str, fname, command, runinfo, nextin, &expand_double_dollar)) {
+                if(!expand_env_to_command(ALLOC &expanded_str, command->mRedirectsFileNames[i], command, runinfo, nextin, &expand_double_dollar)) {
                     return FALSE;
                 }
 
@@ -1389,10 +1483,43 @@ BOOL sCommand_expand_env_redirect(sCommand* command, sObject* nextin, sRunInfo*
                     return FALSE;
                 }
 
-                command->mRedirectsFileNamesRuntime[i] = expanded_str;
+                if(command->mRedirects[i] & REDIRECT_GLOB) {
+                    char* head;
+                    char** tails;
+                    if(!expand_glob(expanded_str, ALLOC &head, ALLOC &tails, command, runinfo->mSName, runinfo->mSLine))
+                    {
+                        FREE(expanded_str);
+                        return FALSE;
+                    }
+                    command->mRedirectsFileNamesRuntime[i] = MANAGED head;
+                    char** p = tails;
+                    while(*p) {
+                        FREE(*p);
+                        p++;
+                    }
+                    FREE(tails);
+                }
+                else {
+                    command->mRedirectsFileNamesRuntime[i] = MANAGED expanded_str;
+                }
+            }
+            else if(command->mRedirects[i] & REDIRECT_GLOB) {
+                char* head;
+                char** tails;
+                if(!expand_glob(command->mRedirectsFileNames[i], ALLOC &head, ALLOC &tails, command, runinfo->mSName, runinfo->mSLine))
+                {
+                    return FALSE;
+                }
+                command->mRedirectsFileNamesRuntime[i] = MANAGED head;
+                char** p = tails;
+                while(*p) {
+                    FREE(*p);
+                    p++;
+                }
+                FREE(tails);
             }
             else {
-                command->mRedirectsFileNamesRuntime[i] = STRDUP(fname);
+                command->mRedirectsFileNamesRuntime[i] = STRDUP(command->mRedirectsFileNames[i]);
             }
         }
     }
