@@ -3,7 +3,6 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <fcntl.h>
-#include <glob.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -223,7 +222,7 @@ static BOOL read_env(char** p, sBuf* buf, char* sname, int* sline, BOOL expand_q
             {
                 FREE(name.mBuf);
                 if(key.mEnv) {  // for readline
-                    sCommand_add_arg_to_command(command, MANAGED key.mBuf, TRUE);
+                    sCommand_add_arg_to_command(command, MANAGED key.mBuf, TRUE, FALSE);
                 }
                 else {
                     FREE(key.mBuf);
@@ -559,7 +558,7 @@ static BOOL read_one_argument(char** p, sBuf* buf, char* sname, int* sline, BOOL
             skip_spaces(p);
         }
         /// glob ///
-        else if(command->mArgsNum > 0 && **p == '[' || **p == '?' || **p == '*') {
+        else if(command->mArgsNum > 0 && (**p == '[' || **p == '?' || **p == '*')) {
             add_char_to_buf(buf, **p);
             (*p)++;
 
@@ -656,40 +655,6 @@ void tilda_expasion(char* file, ALLOC char** file2)
     *file2 = STRDUP(result);
 }
 
-static BOOL glob_expasion(sBuf* buf, ALLOC char** head, ALLOC char*** tails, sCommand* command, char* sname, int* sline)
-{
-    glob_t result;
-    int rc = glob(buf->mBuf, 0, NULL, &result);
-
-    if(rc == GLOB_NOSPACE) {
-        err_msg("read_one_argument: out of space during glob operation", sname, *sline, buf->mBuf);
-        return FALSE;
-    }
-    else if(rc == GLOB_NOMATCH) {
-        *head = STRDUP(buf->mBuf);
-
-        *tails = NULL;
-    }
-    else {
-        if(result.gl_pathc > 0) {
-            *head = STRDUP(result.gl_pathv[0]);
-
-            *tails = MALLOC(sizeof(char*)*result.gl_pathc);
-
-            int i;
-            for(i=1; i<result.gl_pathc; i++) {
-                (*tails)[i-1] = STRDUP(result.gl_pathv[i]);
-            }
-            (*tails)[i-1] = 0;
-        }
-        else {
-            *head = NULL;
-            *tails = NULL;
-        }
-    }
-
-    return TRUE;
-}
 
 static BOOL read_statment(char**p, sStatment* statment, sObject* block, char* sname, int* sline, sObject** current_object);
 
@@ -759,80 +724,21 @@ static BOOL add_argument_to_command(sBuf* buf, sCommand* command, char* sname, i
 
     /// redirect ///
     if(buf->mRedirect) {
+        /// glob ///
         if(buf->mGlob && command->mArgsNum > 0) {
-            char* head;
-            char** globs;
-
-            if(!glob_expasion(buf, ALLOC &head, ALLOC &globs, command, sname, sline)) {
-                if(head) FREE(head);
-                if(globs) {
-                    while(*globs) {
-                        FREE(*globs);
-                        globs++;
-                    }
-                    FREE(globs);
-                }
-                FREE(buf->mBuf);
-                return FALSE;
-            }
-
-            if(head) {
-                sCommand_add_redirect_to_command(command, MANAGED head, buf->mRedirect);
-            }
-
-            if(globs) {
-                char** globs2 = globs;
-                while(*globs2) {
-                    sCommand_add_arg_to_command(command, MANAGED *globs2, 0);
-                    globs2++;
-                }
-
-                FREE(globs);
-            }
-
-            FREE(buf->mBuf);
+            sCommand_add_redirect_to_command(command, MANAGED buf->mBuf, buf->mEnv, buf->mGlob, buf->mRedirect);
         }
         else {
-            sCommand_add_redirect_to_command(command, MANAGED buf->mBuf, buf->mRedirect);
+            sCommand_add_redirect_to_command(command, MANAGED buf->mBuf, buf->mEnv, FALSE, buf->mRedirect);
         }
     }
-    /// expand glob ///
+    /// glob ///
     else if(buf->mGlob && command->mArgsNum > 0) {
-        char* head;
-        char** globs;
-
-        if(!glob_expasion(buf, ALLOC &head, ALLOC &globs, command, sname, sline)) {
-            if(head) FREE(head);
-            if(globs) {
-                while(*globs) {
-                    FREE(*globs);
-                    globs++;
-                }
-                FREE(globs);
-            }
-            FREE(buf->mBuf);
-            return FALSE;
-        }
-
-        if(head) {
-            sCommand_add_arg_to_command(command, MANAGED head, 0);
-        }
-
-        if(globs) {
-            char** globs2 = globs;
-            while(*globs2) {
-                sCommand_add_arg_to_command(command, MANAGED *globs2, 0);
-                globs2++;
-            }
-
-            FREE(globs);
-        }
-
-        FREE(buf->mBuf);
+        sCommand_add_arg_to_command(command, MANAGED buf->mBuf, buf->mEnv, buf->mGlob);
     }
     /// add buf to command ///
     else if(buf->mNullString || buf->mBuf[0] != 0) {
-        sCommand_add_arg_to_command(command, MANAGED buf->mBuf, buf->mEnv);
+        sCommand_add_arg_to_command(command, MANAGED buf->mBuf, buf->mEnv, FALSE);
     }
     else {
         FREE(buf->mBuf);
@@ -853,7 +759,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
         if(!read_one_argument(p, &buf, sname, sline, TRUE, command, block, current_object)) 
         {
             if(buf.mEnv) {  // for readline
-                sCommand_add_arg_to_command(command, MANAGED buf.mBuf, TRUE);
+                sCommand_add_arg_to_command(command, MANAGED buf.mBuf, TRUE, FALSE);
             }
             else {
                 FREE(buf.mBuf);
@@ -888,7 +794,7 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, sObje
                     return FALSE;
                 }
 
-                sCommand_add_arg_to_command(command, MANAGED STRDUP("subshell"), 0);
+                sCommand_add_arg_to_command(command, MANAGED STRDUP("subshell"), 0, 0);
                 sCommand_add_block_to_command(command, block2);
             }
             else {
