@@ -1176,7 +1176,7 @@ static BOOL redirect_ready(sObject** nextout, sObject** nextin, int* opened_fd,
         int result_len;
         int result_size;
 
-        switch(command->mRedirects[i]) {
+        switch(command->mRedirects[i] & REDIRECT_KIND) {
             case REDIRECT_IN:
                 if(SFD(*nextin).mKind == kFDKindBuf) {
                     FREE(SFD(*nextin).fdbuf.mBuf);
@@ -1368,7 +1368,7 @@ static BOOL statment_tree(sStatment* statment, sObject* pipein, sObject* pipeout
         }
 
         if(command->mArgsNum > 0) {
-            if(command->mArgsEnv[0]) {
+            if(command->mArgsFlags[0] & ARG_ENV) {
                 err_msg("a command name must be determined staticaly. Instead of this, use eval inner command.", runinfo->mSName, runinfo->mSLine, NULL);
                 return FALSE;
             }
