@@ -31,11 +31,11 @@ sStatment* sStatment_new(sObject* block, int sline, char* sname);
 void sCommand_delete(sCommand* self);
 void sCommand_delete_malloc(sCommand* self);
 
-void sCommand_add_arg_to_command(sCommand* self, MANAGED char* buf, int env);
+void sCommand_add_arg_to_command(sCommand* self, MANAGED char* buf, int env, int glob);
 BOOL sCommand_add_message(sCommand* self, MANAGED char* message);
 void sCommand_add_env_block(sCommand* self, sObject* block, BOOL double_dollar, enum eLineField lf);
 void sCommand_add_env(sCommand* self, MANAGED char* name, MANAGED char* key, BOOL key_env, BOOL double_dollar);
-void sCommand_add_redirect_to_command(sCommand* self, MANAGED char* name, int redirect);
+void sCommand_add_redirect_to_command(sCommand* self, MANAGED char* name, BOOL env, BOOL glob, int redirect);
 
 sObject* fun_clone_from_stack_block_gc(sObject* block, BOOL user_object, sObject* parent);
 sObject* fun_clone_from_stack_block_stack(sObject* block, sObject* parent);
