@@ -165,9 +165,7 @@ struct sStatment;
 #define STATMENT_ANDAND 0x800000
 #define STATMENT_CONTEXTPIPE_NUMBER 0xFFFF
 
-#define STATMENT_COMMANDS_MAX 32
-#define STATMENT_OPTIONS_MAX 8
-#define STATMENT_OPTION_STRING_MAX 32
+#define STATMENT_COMMANDS_MAX 16
 
 struct block_obj;
 struct _sObject;
@@ -178,7 +176,7 @@ typedef struct _option_hash_it {
 } option_hash_it;
 
 #define XYZSH_OPTION_MAX 32
-#define XYZSH_MESSAGES_MAX 16
+#define XYZSH_MESSAGES_MAX 8
 
 enum eEnvKind { kEnv, kEnvBlock};
 
@@ -199,11 +197,24 @@ typedef struct {
     };
 } sEnv;
 
+#define REDIRECT_KIND 0xff
+
+#define REDIRECT_IN 0x1
+#define REDIRECT_APPEND 0x2
+#define REDIRECT_OUT 0x4
+#define REDIRECT_ERROR_APPEND 0x8
+#define REDIRECT_ERROR_OUT 0x10
+#define REDIRECT_ENV 0x100
+#define REDIRECT_GLOB 0x200
+
+#define ARG_ENV 0x01
+#define ARG_GLOB 0x02
+
 typedef struct {
     enum eCommandKind mKind;
 
     char** mArgs;
-    int* mArgsEnv;    // This index is the same as mArgs, it's true when the variable in the argument will be expanded.
+    int* mArgsFlags;    // This index is the same as mArgs
     int mArgsNum;
     int mArgsSize;
 
@@ -215,6 +226,10 @@ typedef struct {
     int mEnvsNum;
     int mEnvsSize;
 
+    struct _sObject** mBlocks;     // block_obj**
+    int mBlocksNum;
+    int mBlocksSize;
+
     char** mRedirectsFileNames;
     int* mRedirects;
     int mRedirectsNum;
@@ -222,10 +237,6 @@ typedef struct {
 
     char** mRedirectsFileNamesRuntime; //runtime
 
-    struct _sObject** mBlocks;     // block_obj**
-    int mBlocksNum;
-    int mBlocksSize;
-
     option_hash_it mOptions[XYZSH_OPTION_MAX];  // open adressing hash
 
     char* mMessages[XYZSH_MESSAGES_MAX];
@@ -800,12 +811,6 @@ extern sObject* gGlobalPipe;
 int readline_signal();
 void clear_matching_info_variable();
 
-#define REDIRECT_IN 1
-#define REDIRECT_APPEND 2
-#define REDIRECT_OUT 3
-#define REDIRECT_ERROR_APPEND 4
-#define REDIRECT_ERROR_OUT 5
-
 BOOL xyzsh_readline_interface_onetime(int* rcode, char* cmdline, int cursor_point, char* source_name, char** argv, int argc, fXyzshJobDone xyzsh_job_done_);
 char* xyzsh_job_title(int n);
 int xyzsh_job_num();
